import java.io.*;
import java.util.*;

public class TreeInventory
{
  private LinkedList inventory;

   /**
   *  The constructor. It creates an inventory of Trees 
   *  from a file. You do not need to be concerned with 
   *  its implementation.
   *  @String fileName: the name of the data file containing the 
   *  description of the Trees in the InventoryOfTrees.
   */
  public TreeInventory(String fileName)
  {
    String treeDescription;
    StringTokenizer treeDescriptionTokenizer;

    int heightInFeet;
    double priceInDollarsAndCents;
    int qualityFrom1Through10;    

    // Create the LinkedList to hold the inventory of Trees
    inventory = new LinkedList();

    try {

      // Open the file and prepare for reading
      File dataFile = new File (fileName);
      BufferedReader fileData = new 
        BufferedReader(new InputStreamReader (new FileInputStream(dataFile)));

      // Read each entry, creating a new tree. 
      // Stops when it can't get a description line
      while (null != (treeDescription = fileData.readLine()))
      {
        // Break it into the individual descriptors
        treeDescriptionTokenizer = new StringTokenizer (treeDescription);
        heightInFeet = 
          Integer.parseInt(treeDescriptionTokenizer.nextToken());
        priceInDollarsAndCents = 
          Double.parseDouble(treeDescriptionTokenizer.nextToken());
        qualityFrom1Through10 = 
          Integer.parseInt(treeDescriptionTokenizer.nextToken());

        // Create the new tree and add it to the inventory of trees
        inventory.addFirst (new Tree (heightInFeet, priceInDollarsAndCents, 
                                      qualityFrom1Through10));
      }
    }
    catch (EOFException eof)
    {
      // Don't worry -- we're done
    }
    catch (Exception e)
    {
      // This is a real problem
      System.out.println ("Problem reading input file: " +
                           e.toString());
      System.exit (1);
    }
  } 


  /**
    * This is the test program. It takes one paramter: the datafile
    * It will create a TreeInventory from the supplied 
    * inventory file and then sorts it by the priceInDollarsAndCents,
    * from the least expensive to the most expensive. 
    *
    * For example: java TreeInventory datafiles/inventory1.dat 
    *
    * @args[1] is the name of the inventory file, including a path, 
    * if necessary
    */ 
  public static void main (String[] args)
  {
    TreeInventory ti = new TreeInventory (args[0]);
    
    System.out.println ("The following is the initial list of Trees:");
    System.out.println ("");
    System.out.println (ti.inventory);
    System.out.println ("");

    ti.inventory.sort();
    
    System.out.println ("The following is the sorted list of Trees:");
    System.out.println ("");
    System.out.println (ti.inventory);
    System.out.println ("");
   
  }
}
